@@ -122,7 +122,7 @@ namespace zyppng {
       if ( (*iMedia)->_workerType == ProvideQueue::Config::Downloading ) {
         // we keep the information around for an hour so we do not constantly download the media files for no reason
         if ( (*iMedia)->_idleSince && std::chrono::steady_clock::now() - (*iMedia)->_idleSince.value() >= std::chrono::hours(1) ) {
-          MIL << "Detaching medium " << (*iMedia)->_name << " for baseUrl " << (*iMedia)->_attachedUrl << std::endl;
+          MIL << "Detaching medium " << (*iMedia)->_name << " for baseUrl " << (*iMedia)->attachedUrl() << std::endl;
           iMedia = _attachedMediaInfos.erase(iMedia);
           continue;
         } else {
@@ -132,12 +132,12 @@ namespace zyppng {
         // mounting handlers, we need to send a request to the workers
         auto bQueue = (*iMedia)->_backingQueue.lock();
         if ( bQueue ) {
-          zypp::Url url = (*iMedia)->_attachedUrl;
+          zypp::Url url = (*iMedia)->attachedUrl();
           url.setScheme( url.getScheme() + std::string( constants::ATTACHED_MEDIA_SUFFIX) );
           url.setAuthority( (*iMedia)->_name );
           const auto &req = ProvideRequest::createDetach( url );
           if ( req ) {
-            MIL << "Detaching medium " << (*iMedia)->_name << " for baseUrl " << (*iMedia)->_attachedUrl << std::endl;
+            MIL << "Detaching medium " << (*iMedia)->_name << " for baseUrl " << (*iMedia)->attachedUrl() << std::endl;
             bQueue->enqueue ( *req );
             iMedia = _attachedMediaInfos.erase(iMedia);
             continue;
@@ -706,6 +706,36 @@ namespace zyppng {
     return _credManagerOptions;
   }
 
+  std::vector<zypp::Url> ProvidePrivate::sanitizeUrls(const std::vector<zypp::Url> &urls)
+  {
+    std::vector<zypp::Url> usableMirrs;
+    std::optional<ProvideQueue::Config> scheme;
+
+    for ( auto mirrIt = urls.begin() ; mirrIt != urls.end(); mirrIt++ ) {
+      const auto &s = schemeConfig( effectiveScheme( mirrIt->getScheme() ) );
+      if ( !s ) {
+        WAR << "URL: " << *mirrIt << " is not supported, ignoring!" << std::endl;
+        continue;
+      }
+      if ( !scheme ) {
+        scheme = *s;
+        usableMirrs.push_back ( *mirrIt );
+      } else {
+        if ( scheme->worker_type () == s->worker_type () ) {
+          usableMirrs.push_back( *mirrIt );
+        } else {
+          WAR << "URL: " << *mirrIt << " has different worker type than the primary URL: "<< usableMirrs.front() <<", ignoring!" << std::endl;
+        }
+      }
+    }
+
+    if ( !scheme || usableMirrs.empty() ) {
+      return {};
+    }
+
+    return usableMirrs;
+  }
+
   std::vector<AttachedMediaInfo_Ptr> &ProvidePrivate::attachedMediaInfos()
   {
     return _attachedMediaInfos;
@@ -944,9 +974,17 @@ namespace zyppng {
   const zypp::Url &ProvideMediaHandle::baseUrl() const
   {
     static zypp::Url invalidHandle;
+    if ( !_mediaRef || !_mediaRef->_mirrors.size() )
+      return invalidHandle;
+    return _mediaRef->_mirrors.at(0);
+  }
+
+  const std::vector<zypp::Url> &ProvideMediaHandle::mirrors() const
+  {
+    static std::vector<zypp::Url> invalidHandle;
     if ( !_mediaRef )
       return invalidHandle;
-    return _mediaRef->_attachedUrl;
+    return _mediaRef->_mirrors;
   }
 
   const std::optional<zypp::Pathname> &ProvideMediaHandle::localPath() const
@@ -974,6 +1012,37 @@ namespace zyppng {
     return ProvideRef( new Provide(workDir) );
   }
 
+  expected<Provide::LazyMediaHandle> Provide::prepareMedia(const std::vector<zypp::Url> &urls, const ProvideMediaSpec &request)
+  {
+    Z_D();
+    // sanitize the mirrors to contain only URLs that have same worker types
+    std::vector<zypp::Url> usableMirrs = d->sanitizeUrls( urls );
+    if ( usableMirrs.empty() ) {
+      return expected<Provide::LazyMediaHandle>::error( ZYPP_EXCPT_PTR ( zypp::media::MediaException("No valid mirrors available") ));
+    }
+    return expected<Provide::LazyMediaHandle>::success( shared_this<Provide>(), std::move(usableMirrs), request );
+  }
+
+  expected<Provide::LazyMediaHandle> Provide::prepareMedia(const zypp::Url &url, const ProvideMediaSpec &request)
+  {
+    return prepareMedia( std::vector<zypp::Url>{url}, request );
+  }
+
+  AsyncOpRef<expected<Provide::MediaHandle> > Provide::attachMediaIfNeeded( LazyMediaHandle lazyHandle)
+  {
+    using namespace zyppng::operators;
+    if ( lazyHandle.attached() )
+      return makeReadyResult( expected<MediaHandle>::success( *lazyHandle.handle() ) );
+
+    MIL << "Attaching lazy medium with label: [" << lazyHandle.spec().label() << "]" << std::endl;
+
+    return attachMedia( lazyHandle.urls(), lazyHandle.spec () )
+        | and_then([lazyHandle]( MediaHandle handle ) {
+          lazyHandle._sharedData->_mediaHandle = handle;
+          return expected<MediaHandle>::success( std::move(handle) );
+        });
+  }
+
   AsyncOpRef<expected<Provide::MediaHandle>> Provide::attachMedia( const zypp::Url &url, const ProvideMediaSpec &request )
   {
     return attachMedia (  std::vector<zypp::Url>{url}, request );
@@ -983,33 +1052,9 @@ namespace zyppng {
   {
     Z_D();
 
-    if ( urls.empty() ) {
-      return makeReadyResult( expected<Provide::MediaHandle>::error( ZYPP_EXCPT_PTR( zypp::media::MediaException("No usable mirrors in mirrorlist."))) );
-    }
-
     // sanitize the mirrors to contain only URLs that have same worker types
-    std::vector<zypp::Url> usableMirrs;
-    std::optional<ProvideQueue::Config> scheme;
-
-    for ( auto mirrIt = urls.begin() ; mirrIt != urls.end(); mirrIt++ ) {
-      const auto &s = d->schemeConfig( d->effectiveScheme( mirrIt->getScheme() ) );
-      if ( !s ) {
-        WAR << "URL: " << *mirrIt << " is not supported, ignoring!" << std::endl;
-        continue;
-      }
-      if ( !scheme ) {
-        scheme = *s;
-        usableMirrs.push_back ( *mirrIt );
-      } else {
-        if ( scheme->worker_type () == s->worker_type () ) {
-          usableMirrs.push_back( *mirrIt );
-        } else {
-          WAR << "URL: " << *mirrIt << " has different worker type than the primary URL: "<< usableMirrs.front() <<", ignoring!" << std::endl;
-        }
-      }
-    }
-
-    if ( !scheme || usableMirrs.empty() ) {
+    std::vector<zypp::Url> usableMirrs = d->sanitizeUrls( urls );
+    if ( usableMirrs.empty() ) {
       return makeReadyResult( expected<Provide::MediaHandle>::error( ZYPP_EXCPT_PTR ( zypp::media::MediaException("No valid mirrors available") )) );
     }
 
@@ -1047,27 +1092,48 @@ namespace zyppng {
       return makeReadyResult( expected<ProvideRes>::error( ZYPP_EXCPT_PTR( zypp::media::MediaException("Invalid attach handle")) ) );
     }
 
-    // for downloading items we need to make the baseUrl part of the request URL
-    zypp::Url url = (*i)->_attachedUrl;
+    std::vector<zypp::Url> urls;
 
     // real mount devices use a ID to reference a attached medium, for those we do not need to send the baseUrl as well since its already
     // part of the mount point, so if we mount host:/path/to/repo to the ID 1234 and look for the file /path/to/repo/file1 the request URL will look like:  nfs-media://1234/file1
     if ( (*i)->_workerType == ProvideQueue::Config::SimpleMount || (*i)->_workerType == ProvideQueue::Config::VolatileMount ) {
-      url = zypp::Url();
+      auto url = zypp::Url();
       // work around the zypp::Url requirements for certain Url schemes by attaching a suffix, that way we are always able to have a authority
-      url.setScheme( (*i)->_attachedUrl.getScheme() + std::string(constants::ATTACHED_MEDIA_SUFFIX) );
+      url.setScheme( (*i)->attachedUrl().getScheme() + std::string(constants::ATTACHED_MEDIA_SUFFIX) );
       url.setAuthority( (*i)->_name );
       url.setPathName("/");
+      url.appendPathName( fileName );
+      urls.push_back(url);
+    } else {
+
+      // for other items we need to make the baseUrl part of the request URL
+      const auto &addUrl = [&]( const zypp::Url u ){
+        zypp::Url url = u;
+        url.appendPathName( fileName );
+        urls.push_back( url );
+      };
+      std::for_each ( (*i)->_mirrors.begin(), (*i)->_mirrors.end(), addUrl );
     }
 
-    url.appendPathName( fileName );
-    auto op = ProvideFileItem::create( {url}, request, *d );
+    auto op = ProvideFileItem::create( urls, request, *d );
     op->setMediaRef( MediaHandle( *this, (*i) ));
     d->queueItem (op);
 
     return op->promise();
   }
 
+  AsyncOpRef<expected<ProvideRes> > Provide::provide( const LazyMediaHandle &attachHandle, const zypp::Pathname &fileName, const ProvideFileSpec &request )
+  {
+    using namespace zyppng::operators;
+    return attachMediaIfNeeded ( attachHandle )
+    | and_then([weakMe = weak_this<Provide>(), fName = fileName, req = request ]( MediaHandle handle ){
+      auto me = weakMe.lock();
+      if ( !me )
+        return makeReadyResult(expected<ProvideRes>::error(ZYPP_EXCPT_PTR(zypp::Exception("Provide was released during a operation"))));
+      return me->provide( handle, fName, req);
+    });
+  }
+
   zyppng::AsyncOpRef<zyppng::expected<zypp::CheckSum> > Provide::checksumForFile( const zypp::Pathname &p, const std::string &algorithm  )
   {
     using namespace zyppng::operators;
@@ -1080,7 +1146,7 @@ namespace zyppng {
           try {
             return expected<zypp::CheckSum>::success( zypp::CheckSum( algorithm, chksumRes.headers().value(algorithm).asString() ) );
           } catch ( ... ) {
-            return expected<zypp::CheckSum>::error( std::current_exception() );
+            return expected<zypp::CheckSum>::error( ZYPP_FWD_CURRENT_EXCPT() );
           }
         }
         return expected<zypp::CheckSum>::error( ZYPP_EXCPT_PTR( zypp::FileCheckException("Invalid/Empty checksum returned from worker") ) );
