@@ -12,7 +12,6 @@
 #include <zypp/ng/workflows/contextfacade.h>
 #include <iostream>
 #include <vector>
-#include <fstream>
 
 #include <zypp/base/Gettext.h>
 #include <zypp/base/LogTools.h>
@@ -20,7 +19,7 @@
 #include <zypp/parser/xml/XmlEscape.h>
 
 #include <zypp/ManagedFile.h>
-#include <zypp/PublicKey.h>
+#include <zypp-common/PublicKey.h>
 #include <zypp/MediaSetAccess.h>
 #include <zypp/RepoInfo.h>
 #include <zypp/Glob.h>
@@ -43,6 +42,7 @@
 #include <zypp/ZYppCallbacks.h>
 
 #include <zypp/ng/workflows/repoinfowf.h>
+#include <zypp-curl/private/curlhelper_p.h>
 
 using std::endl;
 using zypp::xml::escape;
@@ -84,8 +84,6 @@ namespace zypp
       , _validRepoSignature( indeterminate )
       , _type(repo::RepoType::NONE_e)
       ,	keeppackages(indeterminate)
-      , _mirrorListForceMetalink(false)
-      , emptybaseurls(false)
     {}
 
     Impl(const Impl &) = default;
@@ -142,24 +140,111 @@ namespace zypp
       return ret;
     }
 
+    RepoVariablesReplacedUrl baseUrl() const {
+      if ( !_baseUrls.empty() ) {
+        return RepoVariablesReplacedUrl( _baseUrls.raw ().front(), _baseUrls.transformator() );
+      }
+      if ( !effectiveBaseUrls().empty() ) {
+        return RepoVariablesReplacedUrl( effectiveBaseUrls ().front (), _baseUrls.transformator() );
+      }
+      return RepoVariablesReplacedUrl();
+    }
+
     const RepoVariablesReplacedUrlList & baseUrls() const
     {
-      const Url & mlurl( _mirrorListUrl.transformed() );	// Variables replaced!
-      if ( _baseUrls.empty() && ! mlurl.asString().empty() )
-      {
-        emptybaseurls = true;
-        DBG << "MetadataPath: " << metadataPath() << endl;
-        repo::RepoMirrorList rmurls( mlurl, metadataPath(), _mirrorListForceMetalink );
-        _baseUrls.raw().insert( _baseUrls.raw().end(), rmurls.getUrls().begin(), rmurls.getUrls().end() );
-      }
       return _baseUrls;
     }
 
+    Url location() const {
+      if ( !_baseUrls.empty() )
+        return *_baseUrls.transformedBegin ();
+      return mirrorListUrl().transformed();
+    }
+
+    void resetEffectiveUrls() const {
+      _effectiveBaseUrls.clear ();
+      _lastEffectiveUrlsUpdate = std::chrono::steady_clock::time_point::min();
+    }
+
+    url_set &effectiveBaseUrls() const
+    {
+      if ( !_effectiveBaseUrls.empty()
+           && ( std::chrono::steady_clock::now() - _lastEffectiveUrlsUpdate < std::chrono::hours(1) ) )
+        return _effectiveBaseUrls;
+
+      _effectiveBaseUrls.clear();
+      _lastEffectiveUrlsUpdate = std::chrono::steady_clock::now();
+
+      Url mlurl( mirrorListUrl().transformed() );	// Variables replaced!
+      if ( mlurl.asString().empty()
+           && _baseUrls.raw().size() == 1
+           && repo::RepoMirrorList::urlSupportsMirrorLink( *_baseUrls.transformedBegin() ) ) {
+
+        mlurl = *_baseUrls.transformedBegin ();
+        mlurl.appendPathName("/");
+        mlurl.setQueryParam("mirrorlist", std::string() );
+
+        MIL << "Detected opensuse.org baseUrl with no mirrors, requesting them from : " << mlurl.asString() << std::endl;
+      }
+
+      if ( !mlurl.asString().empty() )
+      {
+        try {
+          DBG << "MetadataPath: " << metadataPath() << endl;
+          repo::RepoMirrorList rmurls( mlurl, metadataPath() );
+
+          // propagate internally used URL params like 'proxy' to the mirrors
+          const auto &tf = [urlTemplate =mirrorListUrl().transformed()]( const zypp::Url &in ){
+            return internal::propagateQueryParams ( in , urlTemplate );
+          };
+          _effectiveBaseUrls.insert( _effectiveBaseUrls.end(), make_transform_iterator( rmurls.getUrls().begin(), tf ), make_transform_iterator( rmurls.getUrls().end(), tf ) );
+        } catch ( const zypp::Exception & e ) {
+          // failed to fetch the mirrorlist/metalink, if we still have a baseUrl we can go on, otherwise this is a error
+          if ( _baseUrls.empty () )
+            throw;
+          else {
+            callback::UserData data( JobReport::repoRefreshMirrorlist );
+            data.set("error", e );
+            JobReport::warning( _("Failed to fetch mirrorlist/metalink data."), data );
+          }
+        }
+      }
+
+      _effectiveBaseUrls.insert( _effectiveBaseUrls.end(), _baseUrls.transformedBegin (), _baseUrls.transformedEnd () );
+      return _effectiveBaseUrls;
+    }
+
+    std::vector<std::vector<Url>> groupedBaseUrls() const
+    {
+      // here we group the URLs to figure out mirrors
+      std::vector<std::vector<Url>> urlGroups;
+      int dlUrlIndex = -1; //we remember the index of the first downloading URL
+
+      const auto &baseUrls = effectiveBaseUrls();
+      std::for_each ( baseUrls.begin(), baseUrls.end(), [&]( const zypp::Url &url ){
+        if ( !url.schemeIsDownloading () ) {
+          urlGroups.push_back ( { url } );
+          return;
+        }
+
+        if ( dlUrlIndex >= 0) {
+          urlGroups[dlUrlIndex].push_back ( url );
+          return;
+        }
+
+        // start a new group
+        urlGroups.push_back ( {url} );
+        dlUrlIndex = urlGroups.size() - 1;
+      });
+
+      return urlGroups;
+    }
+
     RepoVariablesReplacedUrlList & baseUrls()
     { return _baseUrls; }
 
     bool baseurl2dump() const
-    { return !emptybaseurls && !_baseUrls.empty(); }
+    { return !_baseUrls.empty(); }
 
 
     const RepoVariablesReplacedUrlList & gpgKeyUrls() const
@@ -168,6 +253,14 @@ namespace zypp
     RepoVariablesReplacedUrlList & gpgKeyUrls()
     { return _gpgKeyUrls; }
 
+    std::string repoStatusString() const
+    {
+      if ( mirrorListUrl().transformed().isValid() )
+        return mirrorListUrl().transformed().asString();
+      if ( !baseUrls().empty() )
+        return (*baseUrls().transformedBegin()).asString();
+      return std::string();
+    }
 
     const std::set<std::string> & contentKeywords() const
     { hasContent()/*init if not yet done*/; return _keywords.second; }
@@ -324,10 +417,30 @@ namespace zypp
   private:
     TriBool _validRepoSignature; ///< have  signed and valid repo metadata
     repo::RepoType _type;
+
+  private:
+    RepoVariablesReplacedUrl _cfgMirrorlistUrl;
+    RepoVariablesReplacedUrl _cfgMetalinkUrl;
+  public:
+    /** THE mirrorListUrl to work with (either_cfgMirrorlistUrl or _cfgMetalinkUrl) */
+    const RepoVariablesReplacedUrl & mirrorListUrl() const
+    { return _cfgMirrorlistUrl.transformed().isValid() ? _cfgMirrorlistUrl : _cfgMetalinkUrl; }
+
+    void setMirrorlistUrl( const Url & url_r )	// Raw
+    { _cfgMirrorlistUrl.raw() = url_r; }
+
+    void setMetalinkUrl( const Url & url_r )	// Raw
+    { _cfgMetalinkUrl.raw() = url_r; }
+
+    /** Config file writing needs to tell them appart. */
+    const RepoVariablesReplacedUrl & cfgMirrorlistUrl() const
+    { return _cfgMirrorlistUrl; }
+    /** Config file writing needs to tell them appart. */
+    const RepoVariablesReplacedUrl & cfgMetalinkUrl() const
+    { return _cfgMetalinkUrl; }
+
   public:
     TriBool keeppackages;
-    RepoVariablesReplacedUrl _mirrorListUrl;
-    bool                     _mirrorListForceMetalink;
     Pathname path;
     std::string service;
     std::string targetDistro;
@@ -355,14 +468,20 @@ namespace zypp
       return _packagesPath;
     }
 
+    Pathname predownloadPath() const
+    {
+      return packagesPath() / ".preload";
+    }
+
     DefaultIntegral<unsigned,defaultPriority> priority;
-    mutable bool emptybaseurls;
 
   private:
     Pathname _metadataPath;
     Pathname _packagesPath;
 
     mutable RepoVariablesReplacedUrlList _baseUrls;
+    mutable url_set _effectiveBaseUrls;
+    mutable std::chrono::steady_clock::time_point _lastEffectiveUrlsUpdate = std::chrono::steady_clock::time_point::min();
     mutable std::pair<FalseBool, std::set<std::string> > _keywords;
 
     RepoVariablesReplacedUrlList _gpgKeyUrls;
@@ -519,17 +638,26 @@ namespace zypp
     return changed;
   }
 
-  void RepoInfo::setMirrorListUrl( const Url & url_r )	// Raw
-  { _pimpl->_mirrorListUrl.raw() = url_r; _pimpl->_mirrorListForceMetalink = false; }
-
-  void RepoInfo::setMirrorListUrls( url_set urls )	// Raw
-  { setMirrorListUrl( urls.empty() ? Url() : urls.front() ); }
+  void RepoInfo::setMirrorlistUrl( const Url & url_r )	// Raw
+  { _pimpl->setMirrorlistUrl( url_r ); }
 
   void  RepoInfo::setMetalinkUrl( const Url & url_r )	// Raw
-  { _pimpl->_mirrorListUrl.raw() = url_r; _pimpl->_mirrorListForceMetalink = true; }
+  { _pimpl->setMetalinkUrl( url_r ); }
 
+  Url RepoInfo::rawCfgMirrorlistUrl() const
+  { return _pimpl->cfgMirrorlistUrl().raw(); }
+
+  Url RepoInfo::rawCfgMetalinkUrl() const
+  { return _pimpl->cfgMetalinkUrl().raw(); }
+
+#if LEGACY(1735)
+  void RepoInfo::setMirrorListUrl( const Url & url_r )	// Raw
+  { setMirrorlistUrl( url_r ); }
+  void RepoInfo::setMirrorListUrls( url_set urls )	// Raw
+  { _pimpl->setMirrorlistUrl( urls.empty() ? Url() : urls.front() ); }
   void RepoInfo::setMetalinkUrls( url_set urls )	// Raw
-  { setMetalinkUrl( urls.empty() ? Url() : urls.front() ); }
+  { _pimpl->setMetalinkUrl( urls.empty() ? Url() : urls.front() ); }
+#endif
 
   void RepoInfo::setGpgKeyUrls( url_set urls )
   { _pimpl->gpgKeyUrls().raw().swap( urls ); }
@@ -540,27 +668,41 @@ namespace zypp
     _pimpl->gpgKeyUrls().raw().push_back( url_r );
   }
 
-
-  Pathname RepoInfo::provideKey(const std::string &keyID_r, const Pathname &targetDirectory_r) const {
-    return zyppng::RepoInfoWorkflow::provideKey( zyppng::SyncContext::defaultContext(), *this, keyID_r, targetDirectory_r );
-  }
+  std::string RepoInfo::repoStatusString() const
+  { return _pimpl->repoStatusString(); }
 
   void RepoInfo::addBaseUrl( Url url_r )
   {
     for ( const auto & url : _pimpl->baseUrls().raw() )	// Raw unique!
       if ( url == url_r )
         return;
-    _pimpl->baseUrls().raw().push_back( std::move(url_r) );
+
+    _pimpl->baseUrls().raw().push_back( url_r );
+    _pimpl->resetEffectiveUrls ();
   }
 
   void RepoInfo::setBaseUrl( Url url_r )
   {
     _pimpl->baseUrls().raw().clear();
+    _pimpl->resetEffectiveUrls ();
     _pimpl->baseUrls().raw().push_back( std::move(url_r) );
   }
 
   void RepoInfo::setBaseUrls( url_set urls )
-  { _pimpl->baseUrls().raw().swap( urls ); }
+  {
+    _pimpl->resetEffectiveUrls ();
+    _pimpl->baseUrls().raw().swap( urls );
+  }
+
+  bool RepoInfo::effectiveBaseUrlsEmpty() const
+  {
+    return _pimpl->effectiveBaseUrls().empty();
+  }
+
+  RepoInfo::url_set RepoInfo::effectiveBaseUrls() const
+  {
+    return _pimpl->effectiveBaseUrls();
+  }
 
   void RepoInfo::setPath( const Pathname &path )
   { _pimpl->path = path; }
@@ -578,6 +720,9 @@ namespace zypp
   void RepoInfo::setPackagesPath( const Pathname &path )
   { _pimpl->packagesPath( path ); }
 
+  Pathname RepoInfo::predownloadPath() const
+  { return _pimpl->predownloadPath(); }
+
   void RepoInfo::setKeepPackages( bool keep )
   { _pimpl->keeppackages = keep; }
 
@@ -590,6 +735,9 @@ namespace zypp
   bool RepoInfo::keepPackages() const
   { return indeterminate(_pimpl->keeppackages) ? false : (bool)_pimpl->keeppackages; }
 
+  bool RepoInfo::effectiveKeepPackages() const
+  { return keepPackages() || PathInfo(packagesPath().dirname()/".keep_packages").isExist(); }
+
   Pathname RepoInfo::metadataPath() const
   { return _pimpl->metadataPath(); }
 
@@ -603,10 +751,10 @@ namespace zypp
   { return _pimpl->type(); }
 
   Url RepoInfo::mirrorListUrl() const			// Variables replaced!
-  { return _pimpl->_mirrorListUrl.transformed(); }
+  { return _pimpl->mirrorListUrl().transformed(); }
 
   Url RepoInfo::rawMirrorListUrl() const		// Raw
-  { return _pimpl->_mirrorListUrl.raw(); }
+  { return _pimpl->mirrorListUrl().raw(); }
 
   bool RepoInfo::gpgKeyUrlsEmpty() const
   { return _pimpl->gpgKeyUrls().empty(); }
@@ -642,7 +790,13 @@ namespace zypp
   { return _pimpl->targetDistro; }
 
   Url RepoInfo::rawUrl() const
-  { return( _pimpl->baseUrls().empty() ? Url() : *_pimpl->baseUrls().rawBegin() ); }
+  { return _pimpl->baseUrl().raw(); }
+
+  Url RepoInfo::location() const
+  { return _pimpl->location (); }
+
+  std::vector<std::vector<Url>> RepoInfo::groupedBaseUrls() const
+  { return _pimpl->groupedBaseUrls(); }
 
   RepoInfo::urls_const_iterator RepoInfo::baseUrlsBegin() const
   { return _pimpl->baseUrls().transformedBegin(); }
@@ -659,6 +813,11 @@ namespace zypp
   bool RepoInfo::baseUrlSet() const
   { return _pimpl->baseurl2dump(); }
 
+  Url RepoInfo::url() const
+  {
+    return _pimpl->baseUrl().transformed();
+  }
+
   const std::set<std::string> & RepoInfo::contentKeywords() const
   { return _pimpl->contentKeywords(); }
 
@@ -812,7 +971,8 @@ namespace zypp
         str << tag_r << value_r << std::endl;
     });
 
-    strif( (_pimpl->_mirrorListForceMetalink ? "- metalink    : " : "- mirrorlist  : "), rawMirrorListUrl().asString() );
+    strif( "- mirrorlist  : ", _pimpl->cfgMirrorlistUrl().raw().asString() );
+    strif( "- metalink    : ", _pimpl->cfgMetalinkUrl().raw().asString() );
     strif( "- path        : ", path().asString() );
     str << "- type        : " << type() << std::endl;
     str << "- priority    : " << priority() << std::endl;
@@ -845,6 +1005,12 @@ namespace zypp
 
   std::ostream & RepoInfo::dumpAsIniOn( std::ostream & str ) const
   {
+    // libzypp/#638: Add a note to service maintained repo entries
+    if( ! service().empty() ) {
+      str << "# Repository '"<<alias()<<"' is maintained by the '"<<service()<<"' service." << endl;
+      str << "# Manual changes may be overwritten by a service refresh." << endl;
+      str << "# See also 'man zypper', section 'Services'." << endl;
+    }
     RepoInfoBase::dumpAsIniOn(str);
 
     if ( _pimpl->baseurl2dump() )
@@ -861,8 +1027,11 @@ namespace zypp
     if ( ! _pimpl->path.empty() )
       str << "path="<< path() << endl;
 
-    if ( ! (rawMirrorListUrl().asString().empty()) )
-      str << (_pimpl->_mirrorListForceMetalink ? "metalink=" : "mirrorlist=") << hotfix1050625::asString( rawMirrorListUrl() ) << endl;
+    if ( ! _pimpl->cfgMirrorlistUrl().raw().asString().empty() )
+      str << "mirrorlist=" << hotfix1050625::asString( _pimpl->cfgMirrorlistUrl().raw() ) << endl;
+
+    if ( ! _pimpl->cfgMetalinkUrl().raw().asString().empty() )
+      str << "metalink=" << hotfix1050625::asString( _pimpl->cfgMetalinkUrl().raw() ) << endl;
 
     if ( type() != repo::RepoType::NONE )
       str << "type=" << type().asString() << endl;
@@ -920,11 +1089,12 @@ namespace zypp
       str << " raw_repo_gpgcheck=\"" << (_pimpl->rawRepoGpgCheck() ? "1" : "0") << "\"";
     if ( ! indeterminate(_pimpl->rawPkgGpgCheck()) )
       str << " raw_pkg_gpgcheck=\"" << (_pimpl->rawPkgGpgCheck() ? "1" : "0") << "\"";
-    if (!(tmpstr = gpgKeyUrl().asString()).empty())
     if (!(tmpstr = gpgKeyUrl().asString()).empty())
       str << " gpgkey=\"" << escape(tmpstr) << "\"";
-    if (!(tmpstr = mirrorListUrl().asString()).empty())
-      str << (_pimpl->_mirrorListForceMetalink ? " metalink=\"" : " mirrorlist=\"") << escape(tmpstr) << "\"";
+    if ( ! (tmpstr = _pimpl->cfgMirrorlistUrl().transformed().asString()).empty() )
+      str << " mirrorlist=\"" << escape(tmpstr) << "\"";
+    if ( ! (tmpstr = _pimpl->cfgMetalinkUrl().transformed().asString()).empty() )
+      str << " metalink=\"" << escape(tmpstr) << "\"";
     str << ">" << endl;
 
     if ( _pimpl->baseurl2dump() )
@@ -965,7 +1135,8 @@ namespace zypp
   {
     // We skip the check for downloading media unless a local copy of the
     // media file exists and states that there is more than one medium.
-    bool canSkipMediaCheck = std::all_of( baseUrlsBegin(), baseUrlsEnd(), []( const zypp::Url &url ) { return url.schemeIsDownloading(); });
+    const auto &effUrls = _pimpl->effectiveBaseUrls ();
+    bool canSkipMediaCheck = std::all_of( effUrls.begin(), effUrls.end(), []( const zypp::Url &url ) { return url.schemeIsDownloading(); });
     if ( canSkipMediaCheck ) {
       const auto &mDataPath = metadataPath();
       if ( not mDataPath.empty() ) {
