@@ -27,6 +27,7 @@
 #include <zypp/repo/ServiceType.h>
 #include <zypp/ServiceInfo.h>
 #include <zypp/RepoStatus.h>
+#include <zypp/RepoManagerFlags.h>
 #include <zypp/RepoManagerOptions.h>
 #include <utility>
 #include <zypp-core/ui/ProgressData>
@@ -77,30 +78,24 @@ namespace zypp
    /** Dtor */
     ~RepoManager();
 
-    enum RawMetadataRefreshPolicy
-    {
-      RefreshIfNeeded,
-      RefreshForced,
-      RefreshIfNeededIgnoreDelay
-    };
+    using RawMetadataRefreshPolicy = RepoManagerFlags::RawMetadataRefreshPolicy;
+    static constexpr RawMetadataRefreshPolicy RefreshIfNeeded             = RepoManagerFlags::RefreshIfNeeded;
+    static constexpr RawMetadataRefreshPolicy RefreshForced               = RepoManagerFlags::RefreshForced;
+    static constexpr RawMetadataRefreshPolicy RefreshIfNeededIgnoreDelay  = RepoManagerFlags::RefreshIfNeededIgnoreDelay;
 
-    enum CacheBuildPolicy
-    {
-      BuildIfNeeded,
-      BuildForced
-    };
+    using CacheBuildPolicy = RepoManagerFlags::CacheBuildPolicy;
+    static constexpr CacheBuildPolicy BuildIfNeeded = CacheBuildPolicy::BuildIfNeeded;
+    static constexpr CacheBuildPolicy BuildForced   = CacheBuildPolicy::BuildForced;
 
     /** Flags for tuning RefreshService */
-    enum RefreshServiceBit
-    {
-      RefreshService_restoreStatus	= (1<<0),	///< Force restoring repo enabled/disabled status
-      RefreshService_forceRefresh	= (1<<1),	///< Force refresh even if TTL is not reached
-    };
-    ZYPP_DECLARE_FLAGS(RefreshServiceFlags,RefreshServiceBit);
+    using RefreshServiceBit = RepoManagerFlags::RefreshServiceBit;
+    static constexpr RefreshServiceBit RefreshService_restoreStatus = RefreshServiceBit::RefreshService_restoreStatus;
+    static constexpr RefreshServiceBit RefreshService_forceRefresh  = RefreshServiceBit::RefreshService_forceRefresh;
+
+    using RefreshServiceFlags = RepoManagerFlags::RefreshServiceFlags;
 
     /** Options tuning RefreshService */
-    using RefreshServiceOptions = RefreshServiceFlags;
-
+    using RefreshServiceOptions = RepoManagerFlags::RefreshServiceFlags;
 
     /** \name Known repositories.
      *
@@ -142,14 +137,10 @@ namespace zypp
     */
     RepoStatus metadataStatus( const RepoInfo &info ) const;
 
-    /**
-     * Possibly return state of checkIfRefreshMEtadata function
-     */
-    enum RefreshCheckStatus {
-      REFRESH_NEEDED,  /**< refresh is needed */
-      REPO_UP_TO_DATE, /**< repository not changed */
-      REPO_CHECK_DELAYED     /**< refresh is delayed due to settings */
-    };
+    using RefreshCheckStatus = RepoManagerFlags::RefreshCheckStatus;
+    static constexpr RefreshCheckStatus REFRESH_NEEDED     = RefreshCheckStatus::REFRESH_NEEDED;     /**< refresh is needed */
+    static constexpr RefreshCheckStatus REPO_UP_TO_DATE    = RefreshCheckStatus::REPO_UP_TO_DATE;    /**< repository not changed */
+    static constexpr RefreshCheckStatus REPO_CHECK_DELAYED = RefreshCheckStatus::REPO_CHECK_DELAYED; /**< refresh is delayed due to settings */
 
     /**
      * Checks whether to refresh metadata for specified repository and url.
@@ -209,6 +200,18 @@ namespace zypp
                                    const Url &url,
                                    RawMetadataRefreshPolicy policy = RefreshIfNeeded);
 
+    /**
+     * Overload for checkIfToRefreshMetadata using a list of Urls to check if to refresh
+     * metadata. The Urls in the list are treated as mirrors and need to resolve to the
+     * same media handler.
+     *
+     * \sa RepoInfo::groupedBaseUrls
+     *
+     */
+    RefreshCheckStatus checkIfToRefreshMetadata( const RepoInfo &info,
+                                   const std::vector<Url> &url,
+                                   RawMetadataRefreshPolicy policy = RefreshIfNeeded);
+
     /**
      * \short Path where the metadata is downloaded and kept
      *
@@ -246,6 +249,7 @@ namespace zypp
     * \throws repo::RepoNoUrlException if no urls are available.
     * \throws repo::RepoNoAliasException if can't figure an alias
     * \throws repo::RepoUnknownTypeException if the metadata is unknown
+    * \throws repo::RepoNoPermissionException if the user has no permission to update(write) the caches
     * \throws repo::RepoException if the repository is invalid
     *         (no valid metadata found at any of baseurls)
     */
@@ -294,6 +298,7 @@ namespace zypp
     * \throws repo::RepoMetadataException if the metadata
     *     is not enough to build a cache (empty, incorrect, or
     *     refresh needed)
+    * \throws repo::RepoNoPermissionException if the user has no permission to update(write) the caches
     * \throws repo::RepoUnknownTypeException
     * \throws parser::ParseException if parser encounters an error.
     * \throws Exception on unknown error.
@@ -655,7 +660,6 @@ namespace zypp
     /** Pointer to implementation */
     RWCOW_pointer<Impl> _pimpl;
   };
-  ZYPP_DECLARE_OPERATORS_FOR_FLAGS(RepoManager::RefreshServiceFlags);
   ///////////////////////////////////////////////////////////////////
 
   /** \relates RepoManager Stream output */
