@@ -25,6 +25,7 @@ extern "C"
 #include <zypp/base/Logger.h>
 #include <zypp/ExternalProgram.h>
 #include <zypp-media/Mount>
+#include <zypp-media/CDTools>
 #include <zypp/media/MediaCD.h>
 #include <zypp/media/MediaManager.h>
 #include <zypp/Url.h>
@@ -37,13 +38,6 @@ using std::endl;
 */
 #define  REPORT_EJECT_ERRORS     0
 
-/*
-** If defined to the full path of the eject utility,
-** it will be used additionally to the eject-ioctl.
-*/
-#define EJECT_TOOL_PATH "/bin/eject"
-
-
 
 //////////////////////////////////////////////////////////////////
 namespace zypp
@@ -55,7 +49,7 @@ namespace zypp
     //////////////////////////////////////////////////////////////////
     namespace
     {
-      typedef std::list<MediaSource> DeviceList;
+      using DeviceList = std::list<MediaSource>;
 
       //////////////////////////////////////////////////////////////////
       /// \brief Try to detect cd/dvd devices using udev
@@ -133,20 +127,20 @@ namespace zypp
     //////////////////////////////////////////////////////////////////
 
 
-  MediaCD::MediaCD( const Url & url_r, const Pathname & attach_point_hint_r )
-    : MediaHandler( url_r, attach_point_hint_r, url_r.getPathName(), false )
+    MediaCD::MediaCD( MediaUrl url_r, const Pathname & attach_point_hint_r )
+      : MediaHandler( url_r, {}, attach_point_hint_r, url_r.url().getPathName(), false )
     , _lastdev( -1 )
     , _lastdev_tried( -1 )
   {
-    MIL << "MediaCD::MediaCD(" << url_r << ", " << attach_point_hint_r << ")" << endl;
+      MIL << "MediaCD::MediaCD(" << url() << ", " << attach_point_hint_r << ")" << endl;
 
-    if ( url_r.getScheme() != "dvd" && url_r.getScheme() != "cd" )
+      if ( _url.url().getScheme() != "dvd" && _url.url().getScheme() != "cd" )
     {
-      ERR << "Unsupported schema in the Url: " << url_r.asString() << endl;
-      ZYPP_THROW(MediaUnsupportedUrlSchemeException(_url));
+        ERR << "Unsupported schema in the Url: " << _url.url().asString() << endl;
+      ZYPP_THROW(MediaUnsupportedUrlSchemeException(_url.url()));
     }
 
-    std::string devices = _url.getQueryParam( "devices" );
+    std::string devices = _url.url().getQueryParam( "devices" );
     if ( ! devices.empty() )
     {
       std::vector<std::string> words;
@@ -169,8 +163,8 @@ namespace zypp
 
     if ( _devices.empty() )
     {
-      ERR << "Unable to find any cdrom drive for " << _url.asString() << endl;
-      ZYPP_THROW(MediaBadUrlEmptyDestinationException(_url));
+      ERR << "Unable to find any cdrom drive for " << _url.url().asString() << endl;
+      ZYPP_THROW(MediaBadUrlEmptyDestinationException(_url.url()));
     }
   }
 
@@ -182,55 +176,7 @@ namespace zypp
   //
   bool MediaCD::openTray( const std::string & device_r )
   {
-    int fd = ::open( device_r.c_str(), O_RDONLY|O_NONBLOCK|O_CLOEXEC );
-    int res = -1;
-
-    if ( fd != -1)
-    {
-      res = ::ioctl( fd, CDROMEJECT );
-      ::close( fd );
-    }
-
-    if ( res )
-    {
-      if( fd == -1)
-      {
-        WAR << "Unable to open '" << device_r
-            << "' (" << ::strerror( errno ) << ")" << endl;
-      }
-      else
-      {
-        WAR << "Eject " << device_r
-            << " failed (" << ::strerror( errno ) << ")" << endl;
-      }
-
-#if defined(EJECT_TOOL_PATH)
-      DBG << "Try to eject " << device_r << " using "
-        << EJECT_TOOL_PATH << " utility" << std::endl;
-
-      const char *cmd[3];
-      cmd[0] = EJECT_TOOL_PATH;
-      cmd[1] = device_r.c_str();
-      cmd[2] = NULL;
-      ExternalProgram eject(cmd, ExternalProgram::Stderr_To_Stdout);
-
-      for(std::string out( eject.receiveLine());
-          out.length(); out = eject.receiveLine())
-      {
-        DBG << " " << out;
-      }
-
-      if(eject.close() != 0)
-      {
-        WAR << "Eject of " << device_r << " failed." << std::endl;
-        return false;
-      }
-#else
-      return false;
-#endif
-    }
-    MIL << "Eject of " << device_r << " successful." << endl;
-    return true;
+    return CDTools::openTray(device_r);
   }
 
   ///////////////////////////////////////////////////////////////////
@@ -241,19 +187,7 @@ namespace zypp
   //
   bool MediaCD::closeTray( const std::string & device_r )
   {
-    int fd = ::open( device_r.c_str(), O_RDONLY|O_NONBLOCK|O_CLOEXEC );
-    if ( fd == -1 ) {
-      WAR << "Unable to open '" << device_r << "' (" << ::strerror( errno ) << ")" << endl;
-      return false;
-    }
-    int res = ::ioctl( fd, CDROMCLOSETRAY );
-    ::close( fd );
-    if ( res ) {
-      WAR << "Close tray " << device_r << " failed (" << ::strerror( errno ) << ")" << endl;
-      return false;
-    }
-    DBG << "Close tray " << device_r << endl;
-    return true;
+    return CDTools::closeTray(device_r);
   }
 
 
@@ -289,7 +223,7 @@ namespace zypp
       DBG << "creating on-demand device list" << endl;
       //default is /dev/cdrom; for dvd: /dev/dvd if it exists
       std::string device( "/dev/cdrom" );
-      if ( _url.getScheme() == "dvd" && PathInfo( "/dev/dvd" ).isBlk() )
+      if ( _url.url().getScheme() == "dvd" && PathInfo( "/dev/dvd" ).isBlk() )
       {
         device = "/dev/dvd";
       }
@@ -340,12 +274,12 @@ namespace zypp
       ZYPP_THROW(MediaNotSupportedException(url()));
 
     // This also fills the _devices list on demand
-    DeviceList detected( detectDevices( _url.getScheme() == "dvd" ? true : false ) );
+    DeviceList detected( detectDevices( _url.url().getScheme() == "dvd" ? true : false ) );
 
     Mount mount;
     MediaMountException merr;
 
-    std::string options = _url.getQueryParam( "mountoptions" );
+    std::string options = _url.url().getQueryParam( "mountoptions" );
     if ( options.empty() )
     {
       options="ro";
@@ -357,7 +291,7 @@ namespace zypp
     filesystems.push_back("iso9660");
 
     // if DVD, try UDF filesystem after iso9660
-    if ( _url.getScheme() == "dvd" )
+    if ( _url.url().getScheme() == "dvd" )
       filesystems.push_back("udf");
 
     // try all devices in sequence
@@ -520,14 +454,14 @@ namespace zypp
       if( !merr.mountOutput().empty())
       {
         ZYPP_THROW(MediaMountException(merr.mountError(),
-                                       _url.asString(),
+                                       _url.url().asString(),
                                        mountpoint,
                                        merr.mountOutput()));
       }
       else
       {
         ZYPP_THROW(MediaMountException("Mounting media failed",
-                                       _url.asString(), mountpoint));
+                                       _url.url().asString(), mountpoint));
       }
     }
     DBG << _lastdev << " " << count << endl;
@@ -592,7 +526,7 @@ namespace zypp
     if ( ! isAttached() )	// no device mounted in this instance
     {
       // This also fills the _devices list on demand
-      DeviceList detected( detectDevices( _url.getScheme() == "dvd" ? true : false ) );
+      DeviceList detected( detectDevices( _url.url().getScheme() == "dvd" ? true : false ) );
       for_( it, _devices.begin(), _devices.end() )
       {
         MediaSourceRef media( new MediaSource( *it ) );
@@ -722,7 +656,7 @@ namespace zypp
 
     if ( _devices.empty() )
       // This also fills the _devices list on demand
-      detectDevices( _url.getScheme() == "dvd" ? true : false );
+      detectDevices( _url.url().getScheme() == "dvd" ? true : false );
 
     for ( const auto & it : _devices )
       devices.push_back( it.name );
