@@ -22,6 +22,7 @@
 #include <zypp/base/String.h>
 #include <zypp/media/MediaHandler.h>
 #include <zypp/media/MediaManager.h>
+#include <utility>
 #include <zypp-media/Mount>
 #include <limits.h>
 #include <stdlib.h>
@@ -51,17 +52,19 @@ namespace zypp {
 //
 //	DESCRIPTION :
 //
-MediaHandler::MediaHandler ( const Url &      url_r,
-                             const Pathname & attach_point_r,
-                             const Pathname & urlpath_below_attachpoint_r,
-                             const bool       does_download_r )
+  MediaHandler::MediaHandler (MediaUrl primaryUrl_r,
+                              std::vector<MediaUrl> urls_r,
+                              const Pathname & attach_point_r,
+                              Pathname  urlpath_below_attachpoint_r,
+                              const bool       does_download_r )
     : _mediaSource()
     , _attachPoint( new AttachPoint())
     , _attachPointHint()
-    , _relativeRoot( urlpath_below_attachpoint_r)
+    , _relativeRoot(std::move( urlpath_below_attachpoint_r))
     , _does_download( does_download_r )
     , _attach_mtime(0)
-    , _url( url_r )
+    , _urls(std::move( urls_r ))
+    , _url(std::move(primaryUrl_r))
     , _parentId(0)
 {
   Pathname real_attach_point( getRealPath(attach_point_r.asString()));
@@ -80,8 +83,8 @@ MediaHandler::MediaHandler ( const Url &      url_r,
     // schemes other than "file" and "dir", if it is absolute.
     //
     if ( !adir.isDir()
-         || (_url.getScheme() != "file"
-             && _url.getScheme() != "dir"
+         || (_url.url().getScheme() != "file"
+             && _url.url().getScheme() != "dir"
              && !real_attach_point.absolute()) )
     {
       ERR << "Provided attach point is not a absolute directory: "
@@ -120,44 +123,13 @@ MediaHandler::resetParentId()
 std::string
 MediaHandler::getRealPath(const std::string &path)
 {
-  std::string real;
-  if( !path.empty())
-  {
-#if __GNUC__ > 2
-    /** GNU extension */
-    char *ptr = ::realpath(path.c_str(), NULL);
-    if( ptr != NULL)
-    {
-      real = ptr;
-      free( ptr);
-    }
-    else
-    /** the SUSv2 way */
-    if( EINVAL == errno)
-    {
-      char buff[PATH_MAX + 2];
-      memset(buff, '\0', sizeof(buff));
-      if( ::realpath(path.c_str(), buff) != NULL)
-      {
-        real = buff;
-      }
-    }
-#else
-    char buff[PATH_MAX + 2];
-    memset(buff, '\0', sizeof(buff));
-    if( ::realpath(path.c_str(), buff) != NULL)
-    {
-      real = buff;
-    }
-#endif
-  }
-  return real;
+  return getRealPath(zypp::Pathname(path)).asString();
 }
 
 zypp::Pathname
 MediaHandler::getRealPath(const Pathname &path)
 {
-  return zypp::Pathname(getRealPath(path.asString()));
+  return path.realpath();
 }
 
 
@@ -367,7 +339,7 @@ MediaHandler::createAttachPoint() const
   {
     auto except = MediaBadAttachPointException( url() );
     except.addHistory( _("Create attach point: Can't find a writable directory to create an attach point") );
-    ZYPP_THROW( std::move(except) );
+    ZYPP_THROW( except );
   }
 
   MIL << "Created default attach point " << apoint << std::endl;
@@ -533,17 +505,11 @@ MediaHandler::checkAttached(bool matchMountFs) const
         if ( ref.attachPoint->path != Pathname(e->dir) )
           continue;	// at least the mount points must match
 
-        bool        is_device = false;
-        PathInfo    dev_info;
-        if( str::hasPrefix( Pathname(e->src).asString(), "/dev/" ) &&
-            dev_info(e->src) && dev_info.isBlk() )
-        {
-          is_device = true;
-        }
-
+        bool is_device = e->isBlockDevice();
         if( is_device &&  (ref.mediaSource->maj_nr &&
                            ref.mediaSource->bdir.empty()))
         {
+          PathInfo    dev_info(e->src);
           std::string mtype(matchMountFs ? e->type : ref.mediaSource->type);
           MediaSource media(mtype, e->src, dev_info.devMajor(), dev_info.devMinor());
 
@@ -990,7 +956,7 @@ MediaHandler::dependsOnParent(MediaAccessId parentId, bool exactIdMatch)
 //
 //	DESCRIPTION :
 //
-void MediaHandler::provideFileCopy( const OnMediaLocation &srcFile, Pathname targetFilename ) const
+void MediaHandler::provideFileCopy( const OnMediaLocation &srcFile, const Pathname& targetFilename ) const
 {
   if ( !isAttached() ) {
     INT << "Media not_attached on provideFileCopy(" << srcFile
@@ -1022,7 +988,7 @@ void MediaHandler::provideFile( const OnMediaLocation &file ) const
 //
 //	DESCRIPTION :
 //
-void MediaHandler::provideDir( Pathname dirname ) const
+void MediaHandler::provideDir( const Pathname& dirname ) const
 {
   if ( !isAttached() ) {
     INT << "Error: Not attached on provideDir(" << dirname << ")" << endl;
@@ -1041,7 +1007,7 @@ void MediaHandler::provideDir( Pathname dirname ) const
 //
 //	DESCRIPTION :
 //
-void MediaHandler::provideDirTree( Pathname dirname ) const
+void MediaHandler::provideDirTree( const Pathname& dirname ) const
 {
   if ( !isAttached() ) {
     INT << "Error Not attached on provideDirTree(" << dirname << ")" << endl;
@@ -1060,7 +1026,7 @@ void MediaHandler::provideDirTree( Pathname dirname ) const
 //
 //	DESCRIPTION :
 //
-void MediaHandler::releasePath( Pathname pathname ) const
+void MediaHandler::releasePath( const Pathname& pathname ) const
 {
   if ( ! _does_download || _attachPoint->empty() )
     return;
