@@ -14,6 +14,7 @@
 #include <iostream>
 #include <typeinfo>
 
+#include <utility>
 #include <zypp-media/MediaException>
 #include <zypp/media/MediaManager.h>
 #include <zypp/media/MediaHandlerFactory.h>
@@ -53,18 +54,21 @@ namespace zypp
           catch(...) {}
         }
 
-        ManagedMedia( ManagedMedia &&m )
-          : desired ( m.desired )
-          , verifier( std::move(m.verifier) )
-          , _handler ( std::move(m._handler) )
-        {}
+        ManagedMedia(const ManagedMedia &) = delete;
+        ManagedMedia &operator=(const ManagedMedia &) = delete;
 
-        static ManagedMedia makeManagedMedia ( const Url &o_url, const Pathname &preferred_attach_point, const MediaVerifierRef &v )
+        ManagedMedia(ManagedMedia &&m) noexcept
+          : desired(m.desired), verifier(std::move(m.verifier)),
+            _handler(std::move(m._handler)) {}
+
+        static ManagedMedia makeManagedMedia ( const std::vector<MediaUrl> &o_urlList, const Pathname &preferred_attach_point, const MediaVerifierRef &v )
         {
-          auto handler = MediaHandlerFactory::createHandler( o_url, preferred_attach_point );
+          auto handler = MediaHandlerFactory::createHandler( o_urlList, preferred_attach_point );
           if ( !handler ) {
             ERR << "Failed to create media handler" << std::endl;
-            ZYPP_THROW( MediaSystemException(o_url, "Failed to create media handler"));
+            if ( o_urlList.size () )
+              ZYPP_THROW( MediaSystemException( o_urlList.at(0).url(), "Failed to create media handler"));
+            ZYPP_THROW( MediaException("Failed to create media handler") );
           }
           return ManagedMedia( std::move(handler), v );
         }
@@ -161,9 +165,9 @@ namespace zypp
         Pathname         deltafile;
 
       private:
-        ManagedMedia( std::unique_ptr<MediaHandler> &&h, const MediaVerifierRef &v)
+        ManagedMedia( std::unique_ptr<MediaHandler> &&h, MediaVerifierRef v)
           : desired (false)
-          , verifier(v)
+          , verifier(std::move(v))
           , _handler ( std::move(h) )
         {}
 
@@ -175,7 +179,7 @@ namespace zypp
       }
 
       // -------------------------------------------------------------
-      typedef std::map<MediaAccessId, ManagedMedia> ManagedMediaMap;
+      using ManagedMediaMap = std::map<MediaAccessId, ManagedMedia>;
 
       ////////////////////////////////////////////////////////////////
     } // anonymous
@@ -212,13 +216,19 @@ namespace zypp
       {}
 
     public:
+
+      MediaManager_Impl(const MediaManager_Impl &) = delete;
+      MediaManager_Impl(MediaManager_Impl &&) = delete;
+      MediaManager_Impl &operator=(const MediaManager_Impl &) = delete;
+      MediaManager_Impl &operator=(MediaManager_Impl &&) = delete;
+
       ~MediaManager_Impl()
       {
         try
         {
           // remove depending (iso) handlers first
           ManagedMediaMap::iterator it;
-          bool found;
+          bool found = false;
           do
           {
             found = false;
@@ -272,13 +282,7 @@ namespace zypp
       static inline time_t
       getMountTableMTime()
       {
-        time_t mtime = zypp::PathInfo("/etc/mtab").mtime();
-        if( mtime <= 0)
-        {
-          WAR << "Failed to retrieve modification time of '/etc/mtab'"
-              << std::endl;
-        }
-        return mtime;
+        return Mount::getMTime();
       }
 
       static inline MountEntries
@@ -312,10 +316,15 @@ namespace zypp
     // ---------------------------------------------------------------
     MediaAccessId
     MediaManager::open(const Url &url, const Pathname &preferred_attach_point)
+    {
+      return open({MediaUrl(url)}, preferred_attach_point );
+    }
+
+    MediaAccessId MediaManager::open(const std::vector<MediaUrl> &urls, const Pathname &preferred_attach_point)
     {
       // create new access handler for it
       MediaVerifierRef verifier( new NoVerifier());
-      ManagedMedia tmp = ManagedMedia::makeManagedMedia( url, preferred_attach_point, verifier );
+      ManagedMedia tmp = ManagedMedia::makeManagedMedia( urls, preferred_attach_point, verifier );
 
       MediaAccessId nextId = m_impl->nextAccessId();
 
@@ -323,7 +332,7 @@ namespace zypp
       //m_impl->mediaMap[nextId] = std::move(tmp);
 
       DBG << "Opened new media access using id " << nextId
-          << " to " << url.asString() << std::endl;
+          << " to " << urls.at(0).url().asString() << std::endl;
       return nextId;
     }
 
@@ -898,7 +907,7 @@ namespace zypp
         if( ret.mediaSource && ret.attachPoint)
         {
           std::string mnt(ret.attachPoint->path.asString());
-          std::string our(path.asString());
+          const std::string& our(path.asString());
 
           if( our == mnt)
           {
@@ -928,7 +937,7 @@ namespace zypp
       for( e = entries.begin(); e != entries.end(); ++e)
       {
         std::string mnt(Pathname(e->dir).asString());
-        std::string our(path.asString());
+        const std::string& our(path.asString());
 
         if( our == mnt)
         {
