@@ -7,18 +7,23 @@
 |                                                                      |
 \---------------------------------------------------------------------*/
 #include "repodownloaderwf.h"
-#include <zypp/ng/workflows/logichelpers.h>
-#include "zypp/parser/yum/RepomdFileReader.h"
+#include "zypp/ng/reporthelper.h"
+#include <zypp-core/zyppng/pipelines/mtry.h>
+#include <zypp/parser/yum/RepomdFileReader.h>
+#include <zypp/repo/RepoMirrorList.h>
 
 #include <utility>
+#include <fstream>
 #include <zypp-media/ng/Provide>
 #include <zypp-media/ng/ProvideSpec>
 #include <zypp/ng/Context>
 #include <zypp/ng/repo/Downloader>
-#include <zypp/PublicKey.h>
+#include <zypp-common/PublicKey.h>
 #include <zypp/KeyRing.h>
 
 #include <zypp/ng/workflows/signaturecheckwf.h>
+#include <zypp/ng/workflows/repoinfowf.h>
+#include <zypp/ng/workflows/logichelpers.h>
 #include <zypp/ng/repo/workflows/rpmmd.h>
 #include <zypp/ng/repo/workflows/susetags.h>
 #include <zypp/ng/repo/workflows/plaindir.h>
@@ -30,6 +35,7 @@
 #undef  ZYPP_BASE_LOGGER_LOGGROUP
 #define ZYPP_BASE_LOGGER_LOGGROUP "zypp::repomanager"
 
+
 namespace zyppng {
   namespace {
 
@@ -55,33 +61,60 @@ namespace zyppng {
 
     public:
       MaybeAsyncRef<expected<DlContextRefType>> execute( ) {
+
+        zypp::RepoInfo ri = _dlContext->repoInfo();
         // always download them, even if repoGpgCheck is disabled
         _sigpath = _masterIndex.extend( ".asc" );
         _keypath = _masterIndex.extend( ".key" );
         _destdir = _dlContext->destDir();
 
         auto providerRef = _dlContext->zyppContext()->provider();
-        return std::vector {
-           // fetch signature and keys
-            providerRef->provide( _media, _sigpath, ProvideFileSpec().setOptional( true ).setDownloadSize( zypp::ByteCount( 20, zypp::ByteCount::MB ) ) )
-             | and_then( ProvideType::copyResultToDest ( providerRef, _destdir / _sigpath ) ),
-            providerRef->provide( _media, _keypath, ProvideFileSpec().setOptional( true ).setDownloadSize( zypp::ByteCount( 20, zypp::ByteCount::MB ) ) )
-             | and_then( ProvideType::copyResultToDest ( providerRef, _destdir / _keypath ) ),
-           }
-          | join()
-          | [this]( std::vector<expected<zypp::ManagedFile>> &&res ) {
+          return provider()->provide( _media, _masterIndex, ProvideFileSpec().setDownloadSize( zypp::ByteCount( 20, zypp::ByteCount::MB ) ) )
+          | and_then( [this]( ProvideRes && masterres ) {
+            // update the gpg keys provided by the repo
+            return RepoInfoWorkflow::fetchGpgKeys( _dlContext->zyppContext(), _dlContext->repoInfo() )
+            | and_then( [this](){
 
-             // remember downloaded files
-             std::for_each( res.begin (), res.end(),
-               [this]( expected<zypp::ManagedFile> &f){
-                 if (f.is_valid () ) {
-                   _dlContext->files().push_back( std::move(f.get()));
-                 }
-             });
+              // fetch signature and maybe key file
+              return provider()->provide( _media, _sigpath, ProvideFileSpec().setOptional( true ).setDownloadSize( zypp::ByteCount( 20, zypp::ByteCount::MB ) ) )
 
-             // get the master index file
-             return provider()->provide( _media, _masterIndex, ProvideFileSpec().setDownloadSize( zypp::ByteCount( 20, zypp::ByteCount::MB ) ) );
-           }
+              | and_then( ProvideType::copyResultToDest ( provider(), _destdir / _sigpath ) )
+
+              | [this]( expected<zypp::ManagedFile> sigFile ) {
+                  zypp::Pathname sigpathLocal { _destdir/_sigpath };
+                  if ( !sigFile.is_valid () || !zypp::PathInfo(sigpathLocal).isExist() ) {
+                    return makeReadyResult(expected<void>::success()); // no sigfile, valid result
+                  }
+                  _dlContext->files().push_back( std::move(*sigFile) );
+
+                  // check if we got the key, if not we fall back to downloading the .key file
+                  auto expKeyId = mtry( &KeyRing::readSignatureKeyId, _dlContext->zyppContext()->keyRing(), sigpathLocal );
+                  if ( expKeyId && !_dlContext->zyppContext()->keyRing()->isKeyKnown(*expKeyId) ) {
+
+                    if ( _dlContext->repoInfo().mirrorListUrl().isValid() ) {
+                      // when dealing with mirror lists we notify the user to use gpgKeyUrl instead of
+                      // fetching the gpg key from any mirror
+                      JobReportHelper( _dlContext->zyppContext() ).warning(_("Downloading signature key via mirrors, consider explicitely setting gpgKeyUrl via the repository configuration instead."));
+                    }
+
+                    // we did not get the key via gpgUrl downloads, lets fallback
+                    return provider()->provide( _media, _keypath, ProvideFileSpec().setOptional( true ).setDownloadSize( zypp::ByteCount( 20, zypp::ByteCount::MB ) ) )
+                    | and_then( ProvideType::copyResultToDest ( provider(), _destdir / _keypath ) )
+                    | and_then( [this]( zypp::ManagedFile keyFile ) {
+                      _dlContext->files().push_back( std::move(keyFile));
+                      return expected<void>::success();
+                    });
+                  }
+
+                  // we should not reach this line, but if we do we continue and fail later if its required
+                  return makeReadyResult(expected<void>::success());
+                };
+              })
+            | [this,masterres=std::move(masterres)]( expected<void> ) {
+              return make_expected_success( std::move(masterres) );
+            };
+
+          } )
           // execute plugin verification if there is one
           | and_then( std::bind( &DownloadMasterIndexLogic::pluginVerification, this, std::placeholders::_1 ) )
 
@@ -172,8 +205,28 @@ namespace zyppng {
         // The local files are in destdir_r, if they were present on the server
         zypp::Pathname sigpathLocal { _destdir/_sigpath };
         zypp::Pathname keypathLocal { _destdir/_keypath };
+
         if ( _dlContext->pluginRepoverification() && _dlContext->pluginRepoverification()->isNeeded() ) {
           try {
+
+            if ( zypp::PathInfo(sigpathLocal).isExist() && !zypp::PathInfo(keypathLocal).isExist() ) {
+              auto kr = _dlContext->zyppContext()->keyRing();
+              // if we have a signature but no keyfile, we need to export it from the keyring
+              auto expKeyId = mtry( &KeyRing::readSignatureKeyId, kr.get(), sigpathLocal );
+              if ( !expKeyId ) {
+                MIL << "Failed to read signature from file: " << sigpathLocal << std::endl;
+              } else {
+                std::ofstream os( keypathLocal.c_str() );
+                if ( kr->isKeyKnown(*expKeyId) ) {
+                  kr->dumpPublicKey(
+                    *expKeyId,
+                    kr->isKeyTrusted(*expKeyId),
+                    os
+                  );
+                }
+              }
+            }
+
             _dlContext->pluginRepoverification()->getChecker( sigpathLocal, keypathLocal, _dlContext->repoInfo() )( prevRes.file() );
           } catch ( ... ) {
             return expected<ProvideRes>::error( std::current_exception () );
@@ -236,7 +289,7 @@ namespace zyppng {
 
                           auto key = zypp::PublicKey::noThrow( _dlContext->files().back() );
                           if ( not key.fileProvidesKey( keyid ) ) {
-                            const auto &str = zypp::str::Str() << "Keyhint " << file << " does not contain a key with id " << keyid << ". Skipping it.";
+                            const std::string str = (zypp::str::Str() << "Keyhint " << file << " does not contain a key with id " << keyid << ". Skipping it.");
                             WAR << str << std::endl;
                             return makeReadyResult(expected<zypp::PublicKey>::error( std::make_exception_ptr( zypp::Exception(str)) ));
                           }
@@ -415,5 +468,4 @@ namespace zyppng {
         return downloadImpl( dl, std::move(handle), std::move(po) );
       });
     }
-
 }
