@@ -408,7 +408,7 @@ namespace zyppng::RepoManagerWorkflow {
             return makeReadyResult ( expected<RefreshContextRefType>::success( std::move(_refreshContext) ) );
 
           // if REFRESH_NEEDED but we don't have the permission to write the cache, stop here.
-          if ( not zypp::PathInfo(_refreshContext->rawCachePath().dirname()).userMayWX() ) {
+          if ( zypp::IamNotRoot() && not zypp::PathInfo(_refreshContext->rawCachePath().dirname()).userMayWX() ) {
             WAR << "No permision to write cache " << zypp::PathInfo(_refreshContext->rawCachePath().dirname()) << std::endl;
             auto exception = ZYPP_EXCPT_PTR( zypp::repo::RepoNoPermissionException( _refreshContext->repoInfo() ) );
             return makeReadyResult( expected<RefreshContextRefType>::error( std::move(exception) ) );
@@ -504,8 +504,8 @@ namespace zyppng::RepoManagerWorkflow {
       auto helper = std::make_shared<ExHelper>( ExHelper{ refCtx->repoInfo() } );
 
       // the actual logic pipeline, attaches the medium and tries to refresh from it
-      auto refreshPipeline = [ refCtx, progressObserver ]( zypp::Url url ){
-        return refCtx->zyppContext()->provider()->prepareMedia( url, zyppng::ProvideMediaSpec() )
+      auto refreshPipeline = [ refCtx, progressObserver ]( std::vector<zypp::Url> urls ){
+        return refCtx->zyppContext()->provider()->prepareMedia( urls, zyppng::ProvideMediaSpec() )
             | and_then( [ refCtx , progressObserver]( auto mediaHandle ) mutable { return refreshMetadata ( std::move(refCtx), std::move(mediaHandle), progressObserver ); } );
       };
 
@@ -528,8 +528,9 @@ namespace zyppng::RepoManagerWorkflow {
         return true;
       };
 
-      // now go over the urls until we find one that works
-      return refCtx->repoInfo().baseUrls()
+
+      // now go over the url groups until we find one that works
+      return refCtx->repoInfo().groupedBaseUrls()
         | firstOf( std::move(refreshPipeline), expected<RefreshContextRef>::error( std::make_exception_ptr(NotFoundException()) ), std::move(predicate) )
         | [helper]( expected<RefreshContextRef> result ) {
           if ( !result ) {
@@ -679,7 +680,7 @@ namespace zyppng::RepoManagerWorkflow {
             zypp::Pathname mediarootParent { _mediarootpath.dirname() };
 
             if ( zypp::filesystem::assert_dir( mediarootParent ) == 0
-              && zypp::PathInfo(mediarootParent).userMayWX() ) {
+              && ( zypp::IamRoot() || zypp::PathInfo(mediarootParent).userMayWX() ) ) {
 
               return refreshMetadata( _refCtx, ProgressObserver::makeSubTask( _progressObserver ) )
               | and_then([this]( auto /*refCtx*/) { return RepoManager<ZyppContextRefType>::metadataStatus( _refCtx->repoInfo(), _refCtx->repoManagerOptions() ); } );
@@ -748,7 +749,7 @@ namespace zyppng::RepoManagerWorkflow {
             return makeReadyResult( expected<void>::error(ZYPP_EXCPT_PTR(ex)) );
           }
 
-          if( ! zypp::PathInfo(*base).userMayW() )
+          if( zypp::IamNotRoot() && not zypp::PathInfo(*base).userMayW() )
           {
             zypp::Exception ex(zypp::str::form( _("Can't create cache at %s - no writing permissions."), base->c_str()) );
             return makeReadyResult( expected<void>::error(ZYPP_EXCPT_PTR(ex)) );
@@ -1107,7 +1108,7 @@ namespace zyppng::RepoManagerWorkflow {
             return makeReadyResult(expected<void>::success());
           }
 
-          if ( !zypp::PathInfo(_geoIPCache).userMayRWX() ) {
+          if ( zypp::IamNotRoot() && not zypp::PathInfo(_geoIPCache).userMayRWX() ) {
             MIL << "No access rights for the GeoIP cache directory." << std::endl;
             return  makeReadyResult(expected<void>::success());
           }
