@@ -9,12 +9,14 @@
 /** \file	zypp/repo/PackageProvider.cc
  *
 */
+#include "zypp/ng/workflows/contextfacade.h"
 #include <iostream>
 #include <fstream>
 #include <sstream>
 #include <zypp/repo/PackageDelta.h>
 #include <zypp/base/Logger.h>
 #include <zypp/base/Gettext.h>
+#include <utility>
 #include <zypp-core/base/UserRequestException>
 #include <zypp/base/NonCopyable.h>
 #include <zypp/repo/PackageProvider.h>
@@ -32,6 +34,7 @@
 #include <zypp/target/rpm/RpmDb.h>
 #include <zypp/FileChecker.h>
 #include <zypp/target/rpm/RpmHeader.h>
+#include <zypp/ng/workflows/keyringwf.h>
 
 using std::endl;
 
@@ -50,7 +53,7 @@ namespace zypp
     public:
       RpmSigCheckException( repo::DownloadResolvableReport::Action action_r, std::string msg_r = "RpmSigCheckException" )
       : FileCheckException( std::move(msg_r) )
-      , _action( std::move(action_r) )
+      , _action( action_r )
       {}
 
       /** Users final decision how to proceed */
@@ -82,6 +85,10 @@ namespace zypp
     struct PackageProvider::Impl : private base::NonCopyable
     {
       Impl() {}
+      Impl(const Impl &) = delete;
+      Impl(Impl &&) = delete;
+      Impl &operator=(const Impl &) = delete;
+      Impl &operator=(Impl &&) = delete;
       virtual ~Impl() {}
 
       /** Provide the package.
@@ -104,49 +111,54 @@ namespace zypp
     template <class TPackage>
     class PackageProviderImpl : public PackageProvider::Impl
     {
-      typedef typename TPackage::constPtr TPackagePtr;	// Package or SrcPackage
-      typedef callback::UserData UserData;
+      using TPackagePtr = typename TPackage::constPtr;	// Package or SrcPackage
+      using UserData = callback::UserData;
     public:
       /** Ctor taking the Package to provide. */
-      PackageProviderImpl( RepoMediaAccess & access_r, const TPackagePtr & package_r,
-                           const PackageProviderPolicy & policy_r )
-      : _policy( policy_r )
-      , _package( package_r )
+      PackageProviderImpl( RepoMediaAccess &access_r, TPackagePtr &&package_r,
+                           PackageProviderPolicy &&policy_r )
+      : _policy(std::move( policy_r ))
+      , _package(std::move( package_r ))
       , _access( access_r )
       , _retry(false)
       {}
 
-      virtual ~PackageProviderImpl() {}
+      PackageProviderImpl(const PackageProviderImpl &) = delete;
+      PackageProviderImpl(PackageProviderImpl &&) = delete;
+      PackageProviderImpl &operator=(const PackageProviderImpl &) = delete;
+      PackageProviderImpl &operator=(PackageProviderImpl &&) = delete;
+
+      ~PackageProviderImpl() override {}
 
     public:
       /** Provide the package.
        * The basic workflow.
        * \throws Exception.
        */
-      virtual ManagedFile providePackage() const;
+      ManagedFile providePackage() const override;
 
       /** Provide the package if it is cached. */
-      virtual ManagedFile providePackageFromCache() const
+      ManagedFile providePackageFromCache() const override
       {
         ManagedFile ret( doProvidePackageFromCache() );
-        if ( ! ( ret->empty() ||  _package->repoInfo().keepPackages() ) )
+        if ( ! ( ret->empty() || _package->repoInfo().effectiveKeepPackages() ) )
           ret.setDispose( filesystem::unlink );
         return ret;
       }
 
       /** Whether the package is cached. */
-      virtual bool isCached() const
+      bool isCached() const override
       { return ! doProvidePackageFromCache()->empty(); }
 
     protected:
-      typedef PackageProviderImpl<TPackage>	Base;
-      typedef callback::SendReport<repo::DownloadResolvableReport>	Report;
+      using Base = PackageProviderImpl<TPackage>;
+      using Report = callback::SendReport<repo::DownloadResolvableReport>;
 
       /** Lookup the final rpm in cache.
        *
        * A cache hit will return a non empty ManagedFile and an empty one on cache miss.
        *
-       * \note File disposal depending on the repos keepPackages setting
+       * \note File disposal depending on the repos effectiveKeepPackages setting
        * are not set here, but in \ref providePackage or \ref providePackageFromCache.
        */
       ManagedFile doProvidePackageFromCache() const
@@ -234,7 +246,7 @@ namespace zypp
 
               std::string keyID = hr->signatureKeyID();
               if ( keyID.length() > 0 ) {
-                if ( ! getZYpp()->keyRing()->provideAndImportKeyFromRepositoryWorkflow( keyID, info ) )
+                if ( !zyppng::KeyRingWorkflow::provideAndImportKeyFromRepository ( zyppng::SyncContext::defaultContext(), keyID, info ) )
                   break;
 
               } else {
@@ -279,7 +291,7 @@ namespace zypp
         }
       }
 
-      typedef target::rpm::RpmDb RpmDb;
+      using RpmDb = target::rpm::RpmDb;
 
       /** Actual rpm package signature check. */
       RpmDb::CheckPackageResult packageSigCheck( const Pathname & path_r, bool isMandatory_r, UserData & userData ) const
@@ -341,7 +353,7 @@ namespace zypp
       RepoMediaAccess &		_access;
 
     private:
-      typedef shared_ptr<void>	ScopedGuard;
+      using ScopedGuard = shared_ptr<void>;
 
       ScopedGuard newReport() const
       {
@@ -350,8 +362,8 @@ namespace zypp
         // scope (cast required as reset is overloaded). We want report to end
         // when leaving providePackage and not wait for *this going out of scope.
         return shared_ptr<void>( static_cast<void*>(0),
-                                 bind( mem_fun_ref( static_cast<void (shared_ptr<Report>::*)()>(&shared_ptr<Report>::reset) ),
-                                       ref(_report) ) );
+                                 std::bind( std::mem_fn(static_cast<void (shared_ptr<Report>::*)()>(&shared_ptr<Report>::reset)),
+                                            std::ref(_report) ) );
       }
 
       mutable bool               _retry;
@@ -393,7 +405,7 @@ namespace zypp
               if ( filesystem::assert_dir( dest.dirname() ) == 0 && filesystem::hardlinkCopy( pi.path(), dest ) == 0 )
               {
                 ret = ManagedFile( dest );
-                if ( ! info.keepPackages() )
+                if ( ! info.effectiveKeepPackages() )
                   ret.setDispose( filesystem::unlink );
 
                 MIL << "provided Package from toplevel cache " << _package << " at " << ret << endl;
@@ -405,12 +417,11 @@ namespace zypp
         }
       }
 
-      // FIXME we only support the first url for now.
-      if ( info.baseUrlsEmpty() )
+      if ( info.effectiveBaseUrlsEmpty() )
         ZYPP_THROW(Exception("No url in repository."));
 
       MIL << "provide Package " << _package << endl;
-      Url url = * info.baseUrlsBegin();
+      Url url = info.effectiveBaseUrls().front();
       try {
       do {
         _retry = false;
@@ -523,18 +534,18 @@ namespace zypp
     {
     public:
       RpmPackageProvider( RepoMediaAccess & access_r,
-                          const Package::constPtr & package_r,
-                          const DeltaCandidates & deltas_r,
-                          const PackageProviderPolicy & policy_r )
-      : PackageProviderImpl<Package>( access_r, package_r, policy_r )
-      , _deltas( deltas_r )
+                          Package::constPtr &&package_r,
+                          DeltaCandidates &&deltas_r,
+                          PackageProviderPolicy &&policy_r )
+      : PackageProviderImpl<Package>( access_r, std::move(package_r), std::move(policy_r) )
+      , _deltas( std::move(deltas_r) )
       {}
 
     protected:
-      virtual ManagedFile doProvidePackage() const;
+      ManagedFile doProvidePackage() const override;
 
     private:
-      typedef packagedelta::DeltaRpm	DeltaRpm;
+      using DeltaRpm = packagedelta::DeltaRpm;
 
       ManagedFile tryDelta( const DeltaRpm & delta_r ) const;
 
@@ -640,53 +651,54 @@ namespace zypp
     namespace factory
     {
       inline PackageProvider::Impl * make( RepoMediaAccess & access_r, const PoolItem & pi_r,
-                                           const DeltaCandidates & deltas_r,
-                                           const PackageProviderPolicy & policy_r )
+                                           DeltaCandidates &&deltas_r,
+                                           PackageProviderPolicy &&policy_r )
       {
         if ( pi_r.isKind<Package>() )
-          return new RpmPackageProvider( access_r, pi_r->asKind<Package>(), deltas_r, policy_r );
+          return new RpmPackageProvider( access_r, pi_r->asKind<Package>(), std::move(deltas_r), std::move(policy_r) );
         else if ( pi_r.isKind<SrcPackage>() )
-          return new PackageProviderImpl<SrcPackage>( access_r, pi_r->asKind<SrcPackage>(), policy_r );
+          return new PackageProviderImpl<SrcPackage>( access_r, pi_r->asKind<SrcPackage>(), std::move(policy_r) );
         else
           ZYPP_THROW( Exception( str::Str() << "Don't know how to cache non-package " << pi_r.asUserString() ) );
       }
 
       inline PackageProvider::Impl * make( RepoMediaAccess & access_r, const PoolItem & pi_r,
-                                                  const PackageProviderPolicy & policy_r )
+                                                  PackageProviderPolicy &&policy_r )
       {
         if ( pi_r.isKind<Package>() )
-          return new PackageProviderImpl<Package>( access_r, pi_r->asKind<Package>(), policy_r );
+          return new PackageProviderImpl<Package>( access_r, pi_r->asKind<Package>(), std::move(policy_r) );
         else if ( pi_r.isKind<SrcPackage>() )
-          return new PackageProviderImpl<SrcPackage>( access_r, pi_r->asKind<SrcPackage>(), policy_r );
+          return new PackageProviderImpl<SrcPackage>( access_r, pi_r->asKind<SrcPackage>(), std::move(policy_r) );
         else
           ZYPP_THROW( Exception( str::Str() << "Don't know how to cache non-package " << pi_r.asUserString() ) );
       }
 
-      inline PackageProvider::Impl * make( RepoMediaAccess & access_r, const Package::constPtr & package_r,
-                                           const DeltaCandidates & deltas_r,
-                                           const PackageProviderPolicy & policy_r )
-      { return new RpmPackageProvider( access_r, package_r, deltas_r, policy_r ); }
+      inline PackageProvider::Impl * make( RepoMediaAccess & access_r,
+                                           Package::constPtr &&package_r,
+                                           DeltaCandidates &&deltas_r,
+                                           PackageProviderPolicy &&policy_r )
+      { return new RpmPackageProvider( access_r, std::move(package_r), std::move(deltas_r), std::move(policy_r) ); }
 
     } // namespace factory
     ///////////////////////////////////////////////////////////////////
 
     PackageProvider::PackageProvider( RepoMediaAccess & access_r, const PoolItem & pi_r,
-                                      const DeltaCandidates & deltas_r, const PackageProviderPolicy & policy_r )
+                                      DeltaCandidates deltas_r, PackageProviderPolicy policy_r )
 
-    : _pimpl( factory::make( access_r, pi_r, deltas_r, policy_r ) )
+    : _pimpl( factory::make( access_r, pi_r, std::move(deltas_r), std::move(policy_r) ) )
     {}
 
     PackageProvider::PackageProvider( RepoMediaAccess & access_r, const PoolItem & pi_r,
-                                      const PackageProviderPolicy & policy_r )
-    : _pimpl( factory::make( access_r, pi_r, policy_r ) )
+                                      PackageProviderPolicy policy_r )
+    : _pimpl( factory::make( access_r, pi_r, std::move(policy_r) ) )
     {}
 
     /* legacy */
     PackageProvider::PackageProvider( RepoMediaAccess & access_r,
-                                      const Package::constPtr & package_r,
-                                      const DeltaCandidates & deltas_r,
-                                      const PackageProviderPolicy & policy_r )
-    : _pimpl( factory::make( access_r, package_r, deltas_r, policy_r ) )
+                                      Package::constPtr package_r,
+                                      DeltaCandidates deltas_r,
+                                      PackageProviderPolicy policy_r )
+    : _pimpl( factory::make( access_r, std::move(package_r), std::move(deltas_r), std::move(policy_r) ) )
     {}
 
     PackageProvider::~PackageProvider()
