@@ -13,14 +13,15 @@
 #include <fstream>
 #include <sstream>
 #include <streambuf>
-#include <boost/function_output_iterator.hpp>
+#include <boost/iterator/function_output_iterator.hpp>
 
 #define ZYPP_USE_RESOLVER_INTERNALS
 
 #include <zypp/solver/detail/Testcase.h>
 #include <zypp/base/Logger.h>
 #include <zypp/base/LogControl.h>
-#include <zypp/base/GzStream.h>
+#include <utility>
+#include <zypp-core/base/GzStream>
 #include <zypp/base/String.h>
 #include <zypp/base/PtrTypes.h>
 #include <zypp/base/NonCopyable.h>
@@ -31,6 +32,7 @@
 #include <zypp/PathInfo.h>
 #include <zypp/ResPool.h>
 #include <zypp/Repository.h>
+#include <zypp/VendorAttr.h>
 #include <zypp/target/modalias/Modalias.h>
 
 #include <zypp/sat/detail/PoolImpl.h>
@@ -61,8 +63,8 @@ namespace zypp
         :dumpPath("/var/log/YaST2/solverTestcase")
       {}
 
-      Testcase::Testcase(const std::string & path)
-        :dumpPath(path)
+      Testcase::Testcase(std::string  path)
+        :dumpPath(std::move(path))
       {}
 
       Testcase::~Testcase()
@@ -70,6 +72,13 @@ namespace zypp
 
       bool Testcase::createTestcase(Resolver & resolver, bool dumpPool, bool runSolver)
       {
+        // libzypp/issues/317: make sure a satsolver instance is actually present
+        if ( not resolver.get() ) {
+          WAR << "Can't createTestcase if the satsolver is not yet initialized." << endl;
+          return false;
+        }
+
+        MIL << "createTestcase at " << dumpPath << (dumpPool?" dumpPool":"") << (runSolver?" runSolver":"") << endl;
         PathInfo path (dumpPath);
 
         if ( !path.isExist() ) {
@@ -160,8 +169,8 @@ namespace zypp
             yOut << YAML::Value << YAML::BeginMap;
             yOut << YAML::Key << "alias" << YAML::Value << myRepo.alias();
             yOut << YAML::Key << "url" << YAML::BeginSeq;
-            for ( auto itUrl = myRepoInfo.baseUrlsBegin(); itUrl != myRepoInfo.baseUrlsEnd(); ++itUrl ) {
-              yOut << YAML::Value << itUrl->asString();
+            for ( const auto &url : myRepoInfo.effectiveBaseUrls() ) {
+              yOut << YAML::Value << url.asString();
             }
             yOut << YAML::EndSeq;
             yOut << YAML::Key << "path" << YAML::Value << myRepoInfo.path().asString();
@@ -188,14 +197,14 @@ namespace zypp
         const LocaleSet & requestedLocales( satpool.getRequestedLocales() );
 
         yOut << YAML::Key << "locales" << YAML::Value << YAML::BeginSeq ;
-        for ( Locale l : requestedLocales ) {
+        for ( const Locale& l : requestedLocales ) {
           yOut << YAML::Value << YAML::BeginMap;
           yOut << YAML::Key << "fate" << YAML::Value << ( addedLocales.count(l) ? "added" : "" ) ;
           yOut << YAML::Key << "name" << YAML::Value << l.asString() ;
           yOut << YAML::EndMap;
         }
 
-        for ( Locale l : removedLocales ) {
+        for ( const Locale& l : removedLocales ) {
           yOut << YAML::Value << YAML::BeginMap;
           yOut << YAML::Key << "fate" << YAML::Value << "removed" ;
           yOut << YAML::Key << "name" << YAML::Value << l.asString() ;
@@ -203,6 +212,19 @@ namespace zypp
         }
         yOut << YAML::EndSeq; // locales
 
+        // Vendor settings
+        yOut << YAML::Key << "vendors" << YAML::Value << YAML::BeginSeq ;
+        VendorAttr::instance().foreachVendorList( [&]( const VendorAttr::VendorList& vlist )->bool {
+          if ( ! vlist.empty() ) {
+            yOut << YAML::Value << YAML::BeginSeq;
+            for( const auto & v : vlist )
+              yOut << YAML::Value << v ;
+            yOut << YAML::EndSeq;
+          }
+          return true;
+        } );
+        yOut << YAML::EndSeq; // vendors
+
         // helper lambda to write a list of elements into a external file instead of the main file
         const auto &writeListOrFile = [&]( const std::string &name, const auto &list, const auto &callback ) {
           if ( list.size() > 10 ) {
@@ -364,6 +386,7 @@ namespace zypp
         std::ofstream fout( dumpPath+"/zypp-control.yaml" );
         fout << yOut.c_str();
 
+        MIL << "createTestcase done at " << dumpPath << endl;
         return true;
       }
       ///////////////////////////////////////////////////////////////////
